/*  Copyright (C) 2008 National Institute For Space Research (INPE) - Brazil.

    This file is part of the TerraLib - a Framework for building GIS enabled applications.

    TerraLib is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License,
    or (at your option) any later version.

    TerraLib is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with TerraLib. See COPYING. If not, write to
    TerraLib Team at <terralib-team@terralib.org>.
 */

/*!
  \file OutsideFactory.cpp
   
  \brief 

  \ingroup layout
*/

// TerraLib
#include "OutsideFactory.h"
#include "../../../../core/enum/AbstractType.h"
#include "../../../../core/pattern/factory/OutsideParamsCreate.h"
#include "../../../outside/ToolbarOutside.h"
#include "../../../outside/PropertiesOutside.h"
#include "../../../outside/ObjectInspectorOutside.h"
#include "../../../outside/GridSettingsOutside.h"
#include "../../../outside/PageSetupOutside.h"
#include "../../../outside/SystematicScaleOutside.h"
#include "../../../outside/EditTemplateOutside.h"
#include "../../../outside/TextGridSettingsOutside.h"
#include "../../../../core/enum/Enums.h"
#include "../../../outside/MapLayerChoiceOutside.h"

te::layout::OutsideFactory::OutsideFactory()
{

}

te::layout::OutsideFactory::~OutsideFactory()
{

}

te::layout::Observer* te::layout::OutsideFactory::make( EnumType* type, OutsideParamsCreate params /*= LayoutItemParamsCreate()*/ )
{
	Observer* outside = 0;

  EnumObjectType* enumObj = Enums::getInstance().getEnumObjectType();

  if(type == enumObj->getToolbar())
  {
    ToolbarOutside* toolbar = new ToolbarOutside(params.getController(), params.getModel());		
    toolbar->createToolbar();
    outside = (Observer*)toolbar;
  }
  else if(type == enumObj->getPropertiesWindow())
  {
    PropertiesOutside* window = new PropertiesOutside(params.getController(), params.getModel());		
    outside = (Observer*)window;
  }
  else if(type == enumObj->getObjectInspectorWindow())
  {
    ObjectInspectorOutside* inspector = new ObjectInspectorOutside(params.getController(), params.getModel());		
    outside = (Observer*)inspector;
  }
  else if(type == enumObj->getGridSettings())
  {
    GridSettingsOutside* gridSettings = new GridSettingsOutside(params.getController(), params.getModel());		
    outside = (Observer*)gridSettings;
  }
  else if(type == enumObj->getPageSetup())
  {
    PageSetupOutside* pageSetup = new PageSetupOutside(params.getController(), params.getModel());		
    outside = (Observer*)pageSetup;
  }
  else if(type == enumObj->getSystematicScale())
  {
    SystematicScaleOutside* systematic = new SystematicScaleOutside(params.getController(), params.getModel());		
    outside = (Observer*)systematic;
  }
  else if(type == enumObj->getEditTemplate())
  {
    EditTemplateOutside* editTemplate = new EditTemplateOutside(params.getController(), params.getModel());		
    outside = (Observer*)editTemplate;
  }
  else if(type == enumObj->getTextGridSettings())
  {
    TextGridSettingsOutside* textGridSettings = new TextGridSettingsOutside(params.getController(), params.getModel());		
    outside = (Observer*)textGridSettings;
  }
  else if(type == enumObj->getMapLayerChoice())
  {
    MapLayerChoiceOutside* mapLayerChoice = new MapLayerChoiceOutside(params.getController(), params.getModel());		
    outside = (Observer*)mapLayerChoice;
  }

	return outside;
}
